// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

#include "fragmentVersionCentroid.h"

#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		#if defined(TEXEL_AA) && defined(TEXEL_AA_FEATURE)
			_centroid in highp vec2 uv0;
			_centroid in highp vec2 uv1;
		#else
			_centroid in vec2 uv0;
			_centroid in vec2 uv1;
		#endif
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		in vec2 uv0;
		in vec2 uv1;
	#endif
#endif

#include "uniformPerFrameConstants.h"
#include "uniformShaderConstants.h"
#include "util.h"
#include "lib/Hash.h"

varying hmp vec3 cpos;
varying hmp vec3 wpos;

varying vec4 color;

varying float farrender;
varying float farchunk;

#ifdef UNDERWATER
varying float fogr;
#endif

/////////////////////////////////////////////////////////////////
///////////////////////// STOP! /////////////////////////////////
// Made by @bicubic7
// Don't steal anything!
// Don't use code without permit!
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////

#define rain clamp((1.0-pow(FOG_CONTROL.y+.1,10.)),.0,1.)*uv1.y

LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;
LAYOUT_BINDING(1) uniform sampler2D TEXTURE_1;
LAYOUT_BINDING(2) uniform sampler2D TEXTURE_2;

vec3 calclight(vec3 c,float strength){
	vec3 lcolor = mix(vec3(dot(vec3(7.,4.,0)*strength,vec3(1.))),vec3(7.,4.,0)*strength,1.2);
	c += lcolor*pow(1.1*uv1.x,5.5);
return c;
}

//basically from uncharted Tonemap
vec3 Tonemap(vec3 x){
	vec3 A = vec3(1.80, 1.80, 1.80);
	vec3 B = vec3(0.30, 0.30, 0.30);
	vec3 C = vec3(0.35, 0.35, 0.35);
	vec3 D = vec3(0.50, 0.50, 0.50);
	vec3 E = vec3(0.20, 0.20, 0.20);
	vec3 F = vec3(0.30, 0.30, 0.30);
return ((x * (A * x + C * B) + D * E) / (x * (A * x + B) + D * F)) - E / F;
}

float calcWave(highp vec2 pos){
	highp vec2 htime = vec2(0.,TIME);
		pos *= 40.;
	hmp vec2 wp = pos * mat2(vec2(sin(1.),cos(1.8)),vec2(sin(1.),cos(1.)))+(htime * 70.) * 4.0; wp.y *= 2.0;
	hmp vec2 wp1 = pos * mat2(vec2(sin(1.2),cos(0.5)),vec2(sin(.5),cos(0.5)))+(htime * 70.) * 1.5; wp1.y *= 1.0;

	float wave = -noise(wp * 0.005) * 10.0;
		wave += noise(wp1 * 0.01) * 10.0; wave *= 0.2;
return wave;
}

vec4 calcWater(vec4 c,vec4 btex,highp vec2 ttime,highp vec2 ftime){
	vec3 look= normalize(vec3(length(wpos.xz)/4.,-1.,wpos.y))*uv1.y;
	vec3 nw = normalize(vec3(calcWave(cpos.xz*5.)*0.6,calcWave(cpos.xz*5.)*0.65,1.));
	highp vec3 spos = wpos;
	vec4 res = vec4(0);
	vec4 wcolor = mix(vec4(0.,.08,0.2,0.2),vec4(FOG_COLOR.rgb,look.x*1.3),rain);

	float dvalue = mix(.03,mix(.12,.05, 
uv1.y*(1.-ttime.x)),smoothstep(0.75,0.,uv1.y)*(1.-ttime.x)*(1.-rain));

	if(spos.x > 0.) spos -= spos.x*0.45; else spos -= -spos.x*0.45;
	float sm = (1.-rain)*smoothstep(3.0+abs(spos.y)*0.2,0.3,abs(spos.z))*15.,slook = (1.-FOG_COLOR.b)*clamp((FOG_COLOR.r-.15)*1.25,0.,1.);

	wcolor.rgb = mix(wcolor.rgb,mix(calclight(wcolor.rgb,0.12), calclight(wcolor.rgb, dvalue),1.-ttime.x),uv1.x+calcWave(cpos.xz*5.)*1.2);

	float hmap = pow(dot(nw,normalize(vec3(.8,-1.,.8))),3.5);
#ifdef UNDERWATER
		res =  mix(wcolor+clamp(btex.r*look.x*0.3,0.2,0.1),FOG_COLOR, hmap*2.);
#else
		res =  mix(btex.r*wcolor+hmap*FOG_COLOR,FOG_COLOR+min(1.2*slook*ftime.y*3.5,1.)*(sm*hmap)*smoothstep(0.,0.8,uv1.y),0.8*look.x*btex.r);
#endif
		res.a += pow(farchunk,5.);
return res;
}

vec3 ToneColor(vec3 c,vec2 ttime){
	vec3 exposureBias = mix(mix(mix(vec3(2.1, 2.1, 2.1),vec3(2.5,2.2,1.),ttime.y),vec3(2.,2.1,2.5),ttime.x),vec3(1.),rain);

	vec3 curr = Tonemap(exposureBias*c);
return pow(curr/Tonemap(vec3(15.2)),vec3(1.0));
}

vec3 calcFog(vec3 c,vec2 ftime, vec2 ttime){
	vec3 norpos = normalize(wpos);
	if(norpos.x > 0.) norpos -= norpos.x*0.38; else norpos -= -norpos.x*0.38;

	float sunamount = max(0.,1.-length(norpos.zy)), sunlook = (1.-FOG_COLOR.b)*clamp((FOG_COLOR.r-.15)*1.25,0.,1.);

	vec3 fogamount = mix(vec3(0.6,0.8,1.)*FOG_COLOR.rgb,FOG_COLOR.rgb+vec3(FOG_COLOR.r,FOG_COLOR.g,FOG_COLOR.b*2.),sunamount*min(1.2*sunlook*ftime.y*3.5,1.));

return mix(c,fogamount,farrender*mix(0.7,1.3,ttime.y));
}

void main()
{
#ifdef BYPASS_PIXEL_SHADER
	gl_FragColor = vec4(0, 0, 0, 0);
	return;
#else 
	float wf = 0.;
#if USE_TEXEL_AA
	hmp vec4 diffuse = texture2D_AA(TEXTURE_0, uv0);
	vec4 lc = texture2D_AA(TEXTURE_1, vec2(0,1));
#else
	hmp vec4 diffuse = texture2D(TEXTURE_0, uv0);
	vec4 lc = texture2D(TEXTURE_1, vec2(0,1));
#endif

	highp vec2 ttime = vec2(max(1.-min(pow(lc.r*1.8,5.),1.),.0),pow(clamp((1.-pow(lc.r,5.))*1.3,.0,1.),8.))*uv1.y;

	vec2 ftime = vec2(pow(max(min(1.0-FOG_COLOR.r*1.5,1.),.0),1.2),pow(max(min(1.0-FOG_COLOR.r*1.2,1.),.0),.5));

#ifdef SEASONS_FAR
	diffuse.a = 1.0;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
	#define ALPHA_THRESHOLD 0.05
	#else
	#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif

	vec4 inColor = color;

#if defined(BLEND)
	diffuse.a *= inColor.a;
#endif

	vec4 btex = texture2D(TEXTURE_1,uv1);
#if !defined(ALWAYS_LIT)
	diffuse *= btex;
#endif


#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = inColor.a;
	#endif


//reduce default ambient occlusion
if(color.g * 2.0 > color.b + color.r && color.a != 0.){
	diffuse.rgb *= mix(normalize(inColor.rgb),inColor.rgb,0.5);
}else{  
	diffuse.rgb *= color.a ==0.?inColor.rgb: sqrt(inColor.rgb);
}

#else
	vec2 uv = inColor.xy;
	diffuse.rgb *= mix(vec3(1.0,1.0,1.0), texture2D( TEXTURE_2, uv).rgb*2.0, inColor.b);
	diffuse.rgb *= inColor.aaa;
	diffuse.a = 1.0;
#endif


//shadowmapping detection and calculation
	vec3 otex = texture2D(TEXTURE_0, uv0).rgb;
#ifdef FANCY
	vec3 N = normalize(cross(dFdx(cpos.zyx), dFdy(cpos.zyx)));
#endif
	float shadowmap = 0.;
if(color.a==.0){
	shadowmap = mix(shadowmap,.5,max(.0,max(N.z,-N.z)));
	shadowmap = mix(shadowmap,0.8,max(.0,max(N.x,-N.y))*ttime.y*(1.-ttime.x));
} else {
#if !defined(ALPHA_TEST)
	shadowmap = mix(shadowmap,1.,max(.0,max(N.z,-N.z)));
	shadowmap = mix(shadowmap,0.7,max(.0,max(N.x,-N.y))*ttime.y*(1.-ttime.x));
#endif 
}
	shadowmap= mix(shadowmap,0.9,1.0-min(max((uv1.y-0.853)*100.0,0.0),1.0));



//color
	diffuse.rgb = ToneColor(diffuse.rgb,ttime);
	diffuse.rgb *= (diffuse.rgb) + (diffuse.r + diffuse.g + diffuse.b)*(-0.05);

//shadow
	diffuse.rgb *= mix(mix(1.5,1.8,-N.x*ttime.y*(1.-ttime.x)),1.1*otex.g,shadowmap*(1.-max(0.,2.*uv1.x-0.8))*(1.-rain));

//light source color
	float dvalue= mix(mix(.03,mix(.1,.03,uv1.y*(1.-ttime.x)),shadowmap*(1.-ttime.x)),.1,rain);

	diffuse.rgb = mix(diffuse.rgb,mix(calclight(diffuse.rgb,0.1), calclight(diffuse.rgb, dvalue),1.-ttime.x),uv1.x);



#if !defined(SEASONS) && !defined(ALPHA_TEST)
if(color.g < color.b && color.g > color.r){
	diffuse = calcWater(diffuse,btex,ttime, ftime)*mix(1.,.8,shadowmap*(1.-max(0.,2.*uv1.x-0.8))*(1.-rain)); wf =1.;
}
#endif

#ifdef UNDERWATER
if(wf != 1.){
	diffuse = vec4(vec3(.3,.7,1.),.0)*diffuse + max(0.,calcWave(cpos.xz*5.5)*1.2)*uv1.y*vec4(vec3(1.,.95,.8)*1.2,.0)*diffuse;
}
	diffuse.rgb += (1.-uv1.y)*pow(uv1.x*1., 4.)*vec3(1);
//primary fog (only underwater)
	diffuse.rgb = mix(diffuse.rgb,FOG_COLOR.rgb,pow(fogr,7.));
#endif
//secondary fog
	diffuse.rgb = calcFog(diffuse.rgb, ftime, ttime);


//out
	gl_FragColor = diffuse;
#endif // BYPASS_PIXEL_SHADER
}
