#include "ShaderConstants.fxh"
#include "util.fxh"
#include "lib/Hash.fxh"

struct PS_Input
{
	float4 position : SV_Position;
	float3 cpos : CPOS;
	float3 wpos : WPOS;

	float farchunk : FARCHUNK;
	float farrender : FARRENDER;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef UNDERWATER
	float fogr : FOGRENDER;
#endif
};

struct PS_Output
{
	float4 color : SV_Target;
};

/////////////////////////////////////////////////////////////////
///////////////////////// STOP! /////////////////////////////////
// Made by @bicubic7
// Don't steal anything!
// Don't use code without permit!
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////

float3 calclight(float3 c,float2 uv1,float strength){
	float satc = dot(float3(7.,4.,0)*strength,float3(1,1,1));
	float3 lcolor = lerp(float3(satc,satc,satc),float3(7.,4.,0)*strength,1.2);
	c += lcolor*pow(1.1*uv1.x,5.5);
return c;
}

//basically from uncharted Tonemap
float3 Tonemap(float3 x){
	float3 A = float3(1.80, 1.80, 1.80);
	float3 B = float3(0.30, 0.30, 0.30);
	float3 C = float3(0.35, 0.35, 0.35);
	float3 D = float3(0.50, 0.50, 0.50);
	float3 E = float3(0.20, 0.20, 0.20);
	float3 F = float3(0.30, 0.30, 0.30);
return ((x * (A * x + C * B) + D * E) / (x * (A * x + B) + D * F)) - E / F;
}

float calcWave(float2 pos){
	float2 htime = float2(0.,TIME);
		pos *= 40.;
	float2 wp = mul(pos,float2x2(float2(cos(1.8),sin(1.)),float2(cos(1.),sin(1.)))) + (htime * 70.) * 4.0; wp.y *= 2.0;
	float2 wp1 = mul(pos,float2x2(float2(cos(0.5),sin(1.2)),float2(cos(.5),sin(0.5)))) + (htime * 70.) * 1.5; wp1.y *= 1.0;

	float wave = -Noise(wp * 0.005) * 10.0;
		wave += Noise(wp1 * 0.01) * 10.0; wave *= 0.2;
return wave;
}

float4 calcWater(float4 c,float4 btex,float3 wpos,float3 cpos,float2 ttime,float2 ftime,float2 uv1,float rain,float farchunk){
	float3 look= normalize(float3(length(wpos.xz)/4.,-1.,wpos.y))*uv1.y;
	float3 nw = normalize(float3(calcWave(cpos.xz*5.)*0.6,calcWave(cpos.xz*5.)*0.65,1.));
	float3 spos = wpos;
	float4 res = float4(0,0,0,0);
	float4 wcolor = lerp(float4(0.,.08,0.2,0.2),float4(FOG_COLOR.rgb,look.x*1.3),rain);

	float dvalue = lerp(.03,lerp(.12,.05, 
uv1.y*(1.-ttime.x)),smoothstep(0.75,0.,uv1.y)*(1.-ttime.x)*(1.-rain));

	if(spos.x > 0.) spos -= spos.x*0.45; else spos -= -spos.x*0.45;
	float sm = (1.-rain)*smoothstep(3.0+abs(spos.y)*0.2,0.,abs(spos.z))*15.,slook = (1.-FOG_COLOR.b)*clamp((FOG_COLOR.r-.15)*1.25,0.,1.);

	wcolor.rgb = lerp(wcolor.rgb,lerp(calclight(wcolor.rgb,uv1,0.12), calclight(wcolor.rgb,uv1,dvalue),1.-ttime.x),uv1.x+calcWave(cpos.xz*5.)*1.2);

	float hmap = pow(dot(nw,normalize(float3(.8,-1.,.8))),3.5);
#ifdef UNDERWATER
		res =  lerp(wcolor+clamp(btex.r*look.x*0.3,0.2,0.1),FOG_COLOR, hmap*2.);
#else
		res =  lerp(btex.r*wcolor+hmap*FOG_COLOR,FOG_COLOR+min(1.2*slook*ftime.y*3.5,1.)*(sm*hmap)*smoothstep(0.,0.8,uv1.y),0.8*look.x*btex.r);
#endif
		res.a += pow(farchunk,5.);
return res;
}

float3 ToneColor(float3 c,float2 ttime,float rain){
	float3 exposureBias = lerp(lerp(lerp(float3(2.1, 2.1, 2.1),float3(2.5,2.2,1.),ttime.y),float3(2.,2.1,2.5),ttime.x),float3(1,1,1),rain);

	float3 curr = Tonemap(exposureBias*c);
return pow(curr/Tonemap(float3(15.2,15.2,15.2)),float3(1,1,1));
}

float3 calcFog(float3 c,float3 wpos,float2 ftime,float2 ttime,float farrender){
	float3 norpos = normalize(wpos);
	if(norpos.x > 0.) norpos -= norpos.x*0.38; else norpos -= -norpos.x*0.38;

	float sunamount = max(0.,1.-length(norpos.zy)), sunlook = (1.-FOG_COLOR.b)*clamp((FOG_COLOR.r-.15)*1.25,0.,1.);

	float3 fogamount = lerp(float3(0.6,0.8,1.)*FOG_COLOR.rgb,FOG_COLOR.rgb*1.2+float3(FOG_COLOR.r,FOG_COLOR.g,FOG_COLOR.b*2.)*1.2,sunamount*min(1.2*sunlook*ftime.y*3.5,1.));

return lerp(c,fogamount,farrender*lerp(0.7,1.3,ttime.y));
}


ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput)
{
#ifdef BYPASS_PIXEL_SHADER
    PSOutput.color = float4(0.0f, 0.0f, 0.0f, 0.0f);
    return;
#else
	float wf = 0.;
#if USE_TEXEL_AA
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv0);
	float4 lc = texture2D_AA(TEXTURE_1, TextureSampler1, float2(0.0, 1.0));
#else
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);
	float4 lc = TEXTURE_1.Sample(TextureSampler1, float2(0.0, 1.0));
#endif

	float2 ttime = float2(max(1.-min(pow(lc.r*1.8,5.),1.),.0),pow(clamp((1.-pow(lc.r,5.))*1.3,.0,1.),8.))*PSInput.uv1.y;
	float2 ftime = float2(pow(max(min(1.0-FOG_COLOR.r*1.5,1.),.0),1.2),pow(max(min(1.0-FOG_COLOR.r*1.2,1.),.0),.5));

#ifdef SEASONS_FAR
	diffuse.a = 1.0f;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
		#define ALPHA_THRESHOLD 0.05
	#else
		#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif

#if defined(BLEND)
	diffuse.a *= PSInput.color.a;
#endif
	float4 btex = TEXTURE_1.Sample(TextureSampler1, PSInput.uv1);
#if !defined(ALWAYS_LIT)
	diffuse = diffuse * btex;
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = PSInput.color.a;
	#endif	
	float4 inColor = PSInput.color;

	//reduce default ambient occlusion
if(PSInput.color.g * 2.0 > PSInput.color.b + PSInput.color.r && PSInput.color.a != 0.){
	diffuse.rgb *= lerp(normalize(inColor.rgb),inColor.rgb,0.5);
}else{  
	diffuse.rgb *= PSInput.color.a ==0.?inColor.rgb: sqrt(inColor.rgb);
}

#else
	float2 uv = PSInput.color.xy;
	diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, uv).rgb*2.0f, PSInput.color.b);
	diffuse.rgb *= PSInput.color.aaa;
	diffuse.a = 1.0f;
#endif
	float rain = 1.-pow(FOG_CONTROL.y,10.);
//shadowmapping detection and calculation
	float3 otex = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0).rgb;
#ifdef FANCY
	float3 N = normalize(cross(ddx(-PSInput.cpos.zyx), ddy(PSInput.cpos.zyx)));
#endif
	float shadowmap = 0.;
if(PSInput.color.a==.0){
	shadowmap = lerp(shadowmap,.5,max(.0,max(N.z,-N.z)));
	shadowmap = lerp(shadowmap,0.8,max(.0,max(N.x,-N.y))*ttime.y*(1.-ttime.x));
} else {
#if !defined(ALPHA_TEST)
	shadowmap = lerp(shadowmap,1.,max(.0,max(N.z,-N.z)));
	shadowmap = lerp(shadowmap,0.8,max(.0,max(N.x,-N.y))*ttime.y*(1.-ttime.x));
#endif 
}
	shadowmap= lerp(shadowmap,0.9,1.0-min(max((PSInput.uv1.y-0.853)*100.0,0.0),1.0));



//color
	diffuse.rgb = ToneColor(diffuse.rgb,ttime,rain);
	diffuse.rgb *= (diffuse.rgb) + (diffuse.r + diffuse.g + diffuse.b)*(-0.05);

//shadow
	diffuse.rgb *= lerp(lerp(1.5,1.8,-N.x*ttime.y*(1.-ttime.x)),1.1*otex.g,shadowmap*(1.-max(0.,2.*PSInput.uv1.x-0.8))*(1.-rain));

//light source color
	float dvalue= lerp(lerp(.02,lerp(.1,.03,PSInput.uv1.y*(1.-ttime.x)),shadowmap*(1.-ttime.x)),.1,rain);
	diffuse.rgb = lerp(diffuse.rgb,lerp(calclight(diffuse.rgb,PSInput.uv1,0.1), calclight(diffuse.rgb,PSInput.uv1,dvalue),1.-ttime.x),PSInput.uv1.x);



#if !defined(SEASONS) && !defined(ALPHA_TEST)
if(PSInput.color.g < PSInput.color.b && PSInput.color.g > PSInput.color.r){
	diffuse = calcWater(diffuse,btex,PSInput.wpos,PSInput.cpos,ttime,ftime,PSInput.uv1,rain,PSInput.farchunk)*lerp(1.,.8,shadowmap*(1.-max(0.,2.*PSInput.uv1.x-0.8))*(1.-rain)); wf =1.;
}
#endif

#ifdef UNDERWATER
if(wf != 1.){
	diffuse = float4(float3(.3,.7,1.),.0)*diffuse + max(0.,calcWave(PSInput.cpos.xz*5.5)*1.2)*PSInput.uv1.y*float4(float3(1.,.95,.8)*1.2,.0)*diffuse;
}
	diffuse.rgb += (1.-PSInput.uv1.y)*pow(PSInput.uv1.x*1., 4.)*float3(1,1,1);
//primary fog (only underwater)
	diffuse.rgb = lerp(diffuse.rgb, FOG_COLOR.rgb, pow(PSInput.fogr,7.));
#endif
//secondary fog
	diffuse.rgb = calcFog(diffuse.rgb,PSInput.wpos,ftime, ttime,PSInput.farrender);

	PSOutput.color = diffuse;

#ifdef VR_MODE
	// On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to 
	// the lowest 8 bit value.
	PSOutput.color = max(PSOutput.color, 1 / 255.0f);
#endif

#endif // BYPASS_PIXEL_SHADER
}