#include "ShaderConstants.fxh"
#include "lib/Hash.fxh"
struct VS_Input {
	float3 position : POSITION;
	float4 color : COLOR;
	float2 uv0 : TEXCOORD_0;
	float2 uv1 : TEXCOORD_1;
#ifdef INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
};

struct PS_Input {
	float4 position : SV_Position;
	float3 cpos : CPOS;
	float3 wpos : WPOS;

	float farchunk : FARCHUNK;
	float farrender : FARRENDER;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef UNDERWATER
	float fogr : FOGRENDER;
#endif
#ifdef GEOMETRY_INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
#ifdef VERTEXSHADER_INSTANCEDSTEREO
	uint renTarget_id : SV_RenderTargetArrayIndex;
#endif
};


static const float rA = 1.0;
static const float rB = 1.0;
static const float3 UNIT_Y = float3(0, 1, 0);
static const float DIST_DESATURATION = 56.0 / 255.0; //WARNING this value is also hardcoded in the water color, don'tchange


ROOT_SIGNATURE
void main(in VS_Input VSInput, out PS_Input PSInput)
{
	
#ifndef BYPASS_PIXEL_SHADER
	PSInput.uv0 = VSInput.uv0;
	PSInput.uv1 = VSInput.uv1;
	PSInput.color = VSInput.color;
#endif

#ifdef AS_ENTITY_RENDERER
	#ifdef INSTANCEDSTEREO
		int i = VSInput.instanceID;
		PSInput.position = mul(WORLDVIEWPROJ_STEREO[i], float4(VSInput.position, 1));
	#else
		PSInput.position = mul(WORLDVIEWPROJ, float4(VSInput.position, 1));
	#endif
		float3 worldPos = PSInput.position;
#else
		float3 worldPos = (VSInput.position.xyz * CHUNK_ORIGIN_AND_SCALE.w) + CHUNK_ORIGIN_AND_SCALE.xyz;
	
		// Transform to view space before projection instead of all at once to avoid floating point errors
		// Not required for entities because they are already offset by camera translation before rendering
		// World position here is calculated above and can get huge
	#ifdef INSTANCEDSTEREO
		int i = VSInput.instanceID;
	
		PSInput.position = mul(WORLDVIEW_STEREO[i], float4(worldPos, 1 ));
		PSInput.position = mul(PROJ_STEREO[i], PSInput.position);
	
	#else
		PSInput.position = mul(WORLDVIEW, float4( worldPos, 1 ));
		PSInput.position = mul(PROJ, PSInput.position);
	#endif
#endif

	PSInput.cpos = VSInput.position.xyz;
	PSInput.wpos = worldPos.xyz;

	PSInput.farrender = length(worldPos.xyz)/RENDER_DISTANCE;
	PSInput.farchunk = length(worldPos.xyz)/FAR_CHUNKS_DISTANCE;
	float3 wapos =float3(VSInput.position.x==16.0 ? 0.0 :VSInput.position.x, abs(VSInput.position.y-8.0),VSInput.position.z==16.0 ? 0.0 : VSInput.position.z);

	float wave_factor = 0.0;
	
if(VSInput.color.a == 0.0){
	wave_factor = 0.2*(1.0-length(worldPos.xyz)/FAR_CHUNKS_DISTANCE);
	wave_factor *= VSInput.uv1.y;
}else{

#ifdef ALPHA_TEST
if(VSInput.color.b < VSInput.color.g){
	wave_factor = 0.2*(1.0-length(worldPos.xyz)/FAR_CHUNKS_DISTANCE);
	wave_factor *= VSInput.uv1.y;
}
#endif
}

#ifdef UNDERWATER
	wave_factor = 0.01*pow(PSInput.position.z,1.5);
#endif

if(wave_factor > 0.0){
	float wave = sin(TIME*3.+wapos.x+wapos.z+wapos.y)*hash(wapos.x+wapos.y+wapos.z);
	PSInput.position.x += wave * 0.2 * wave_factor;
}

#ifdef GEOMETRY_INSTANCEDSTEREO
		PSInput.instanceID = VSInput.instanceID;
#endif 
#ifdef VERTEXSHADER_INSTANCEDSTEREO
		PSInput.renTarget_id = VSInput.instanceID;
#endif

	float3 relPos = -worldPos;
	float cameraDepth = length(relPos);

#ifdef UNDERWATER
	float len = cameraDepth / RENDER_DISTANCE;
#ifdef ALLOW_FADE
	len += RENDER_CHUNK_FOG_ALPHA.r;
#endif
	PSInput.fogr= clamp((len - FOG_CONTROL.x) / (FOG_CONTROL.y - FOG_CONTROL.x), 0.0, 1.0);
#endif

}
